/**
 * @file
 * VuoWindowOpenGLInternal interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOWINDOWOPENGLINTERNAL_HH
#define VUOWINDOWOPENGLINTERNAL_HH

#include <QMainWindow>
#include <QGLWidget>
#include <QOpenGLContext>
#include <QShortcut>

#include <ApplicationServices/ApplicationServices.h>

#include "node.h"
#include "VuoGlContext.h"

/**
 * Private implementation of @c VuoWindowOpenGL.
 */
class VuoWindowOpenGLInternal : public QMainWindow
{
	Q_OBJECT

private:
	void activateGLContext(void);
	void deactivateGLContext(void);
	void render(void);

	bool callbacksEnabled;
	void (*initCallback)(void *);
	void (*resizeCallback)(void *, unsigned int width, unsigned int height);
	void (*drawCallback)(void *);
	void *drawContext;

	void (*frameRequested)(VuoInteger); /// @todo change to struct...
	void (*mouseMoved)(VuoPoint2d); /// @todo implement
	void (*mouseLeftPressed)(VuoPoint2d); /// @todo implement

	VuoGlContext windowedGlContext;
	VuoGlContext fullScreenGlContext;
	QGLWidget * glWidget(void);
	QOpenGLContext * glContext(void);

	bool actualAspectDiffersSignificantlyFromIdeal(void);
	float aspect;
	bool resizingWindow;
	bool programmaticallyResizingWindow;
	bool userResizedWindow;

	bool isFullScreen;
	QAction *fullScreenAction;
	QShortcut *exitFullScreenShortcut;
	CGDirectDisplayID fullScreenDisplay;
	void createFullScreenGLContext(void);
	void destroyFullScreenContext(void);
	void setFullScreenAspectRatio(void);

private slots:
	void toggleFullScreen();

public:
	VuoWindowOpenGLInternal
	(
			void (*initCallback)(void *),
			void (*resizeCallback)(void *, unsigned int width, unsigned int height),
			void (*drawCallback)(void *),
			void *drawContext
	);
	void enableTriggers
	(
			VuoOutputTrigger(frameRequested, VuoInteger), /// @todo change to struct...
			VuoOutputTrigger(mouseMoved, VuoPoint2d),
			VuoOutputTrigger(mouseLeftPressed, VuoPoint2d)
	);
	void disableTriggers(void);
	void scheduleRedraw(void);
	void setAspectRatio(unsigned int pixelsWide, unsigned int pixelsHigh);

protected:
	bool event(QEvent *event);
	void resizeEvent(QResizeEvent *event);
};

#endif // VUOWINDOWOPENGLINTERNAL_HH
